@@ -1,7 +1,7 @@
 use crate::{
     ast::{
-        Arg, Definition, Import, ModuleConstant, Publicity, SrcSpan, TypedDefinition, TypedExpr,
-        TypedFunction, TypedModule, TypedPattern,
+        Arg, Definition, ModuleConstant, SrcSpan, TypedExpr, TypedFunction, TypedModule,
+        TypedPattern,
     },
     build::{type_constructor_from_modules, Located, Module, UnqualifiedImport},
     config::PackageConfig,
@@ -11,10 +11,7 @@ use crate::{
     },
     line_numbers::LineNumbers,
     paths::ProjectPaths,
-    type_::{
-        pretty::Printer, ModuleInterface, PreludeType, Type, TypeConstructor,
-        ValueConstructorVariant,
-    },
+    type_::{pretty::Printer, ModuleInterface, Type, TypeConstructor, ValueConstructorVariant},
     Error, Result, Warning,
 };
 use camino::Utf8PathBuf;
@@ -22,10 +19,10 @@ use ecow::EcoString;
 use lsp::CodeAction;
 use lsp_types::{self as lsp, Hover, HoverContents, MarkedString, Url};
 use std::sync::Arc;
-use strum::IntoEnumIterator;
 
 use super::{
     code_action::{CodeActionBuilder, RedundantTupleInCaseSubject},
+    completer::Completer,
     src_span_to_lsp_range, DownloadDependencies, MakeLocker,
 };
 
@@ -144,8 +141,6 @@ where
         self.compiler.take_warnings()
     }
 
-    // TODO: test different package module function calls
-    //
     // TODO: implement unqualified imported module functions
     //
     pub fn goto_definition(
@@ -195,45 +190,53 @@ where
                 None => return Ok(None),
             };
 
+            let completer = Completer::new(&src, &params, &this.compiler, module);
+
             // Check current filercontents if the user is writing an import
             // and handle separately from the rest of the completion flow
             // Check if an import is being written
-            if let Some(value) = this.import_completions(src, &params, module) {
+            if let Some(value) = completer.import_completions() {
                 return value;
             }
 
-            let line_numbers = LineNumbers::new(&module.code);
-            let byte_index =
-                line_numbers.byte_index(params.position.line, params.position.character);
+            let byte_index = completer
+                .module_line_numbers
+                .byte_index(params.position.line, params.position.character);
+
+            // If in comment context, do not provide completions
+            if module.extra.is_within_comment(byte_index) {
+                return Ok(None);
+            }
 
             let Some(found) = module.find_node(byte_index) else {
                 return Ok(None);
             };
 
             let completions = match found {
+                Located::PatternSpread { .. } => None,
                 Located::Pattern(_pattern) => None,
 
                 Located::Statement(_) | Located::Expression(_) => {
-                    Some(this.completion_values(module))
+                    Some(completer.completion_values())
                 }
 
                 Located::ModuleStatement(Definition::Function(_)) => {
-                    Some(this.completion_types(module))
+                    Some(completer.completion_types())
                 }
 
-                Located::FunctionBody(_) => Some(this.completion_values(module)),
+                Located::FunctionBody(_) => Some(completer.completion_values()),
 
                 Located::ModuleStatement(Definition::TypeAlias(_) | Definition::CustomType(_)) => {
-                    Some(this.completion_types(module))
+                    Some(completer.completion_types())
                 }
 
                 // If the import completions returned no results and we are in an import then
                 // we should try to provide completions for unqualified values
                 Located::ModuleStatement(Definition::Import(import)) => this
                     .compiler
-                    .get_module_inferface(import.module.as_str())
+                    .get_module_interface(import.module.as_str())
                     .map(|importing_module| {
-                        this.unqualified_completions_from_module(importing_module, module, true)
+                        completer.unqualified_completions_from_module(importing_module)
                     }),
 
                 Located::ModuleStatement(Definition::ModuleConstant(_)) => None,
@@ -242,14 +245,17 @@ where
 
                 Located::Arg(_) => None,
 
-                Located::Annotation(_, _) => Some(this.completion_types(module)),
+                Located::Annotation(_, _) => Some(completer.completion_types()),
             };
 
             Ok(completions)
         })
     }
 
-    pub fn action(&mut self, params: lsp::CodeActionParams) -> Response<Option<Vec<CodeAction>>> {
+    pub fn code_actions(
+        &mut self,
+        params: lsp::CodeActionParams,
+    ) -> Response<Option<Vec<CodeAction>>> {
         self.respond(|this| {
             let mut actions = vec![];
             let Some(module) = this.module_for_uri(&params.text_document.uri) else {
@@ -310,7 +316,7 @@ where
                     location,
                 }) => this
                     .compiler
-                    .get_module_inferface(module.as_str())
+                    .get_module_interface(module.as_str())
                     .and_then(|module| {
                         if is_type {
                             module.types.get(name).map(|t| {
@@ -328,6 +334,48 @@ where
                         }
                     }),
                 Located::Pattern(pattern) => Some(hover_for_pattern(pattern, lines)),
+                Located::PatternSpread {
+                    spread_location,
+                    arguments,
+                } => {
+                    let range = Some(src_span_to_lsp_range(spread_location, &lines));
+
+                    let mut positional = vec![];
+                    let mut labelled = vec![];
+                    for argument in arguments {
+                        // We only want to display the arguments that were ignored using `..`.
+                        // Any argument ignored that way is marked as implicit, so if it is
+                        // not implicit we just ignore it.
+                        if !argument.implicit {
+                            continue;
+                        }
+
+                        let type_ = Printer::new().pretty_print(argument.value.type_().as_ref(), 0);
+                        match &argument.label {
+                            Some(label) => labelled.push(format!("- `{}: {}`", label, type_)),
+                            None => positional.push(format!("- `{}`", type_)),
+                        }
+                    }
+
+                    let positional = positional.join("\n");
+                    let labelled = labelled.join("\n");
+                    let content = match (positional.is_empty(), labelled.is_empty()) {
+                        (true, false) => format!("Unused labelled fields:\n{labelled}"),
+                        (false, true) => format!("Unused positional fields:\n{positional}"),
+                        (_, _) => format!(
+                            "Unused positional fields:
+{positional}
+
+Unused labelled fields:
+{labelled}"
+                        ),
+                    };
+
+                    Some(Hover {
+                        contents: HoverContents::Scalar(MarkedString::from_markdown(content)),
+                        range,
+                    })
+                }
                 Located::Expression(expression) => {
                     let module = this.module_for_uri(&params.text_document.uri);
 
@@ -399,370 +447,6 @@ where
 
         self.compiler.modules.get(&module_name)
     }
-
-    /// checks based on the publicity if something should be suggested for import from root package
-    fn is_suggestable_import(&self, publicity: &Publicity, package: &str) -> bool {
-        match publicity {
-            // We skip private types as we never want those to appear in
-            // completions.
-            Publicity::Private => false,
-            // We only skip internal types if those are not defined in
-            // the root package.
-            Publicity::Internal if package != self.root_package_name() => false,
-            Publicity::Internal => true,
-            // We never skip public types.
-            Publicity::Public => true,
-        }
-    }
-
-    fn completion_types<'b>(&'b self, module: &'b Module) -> Vec<lsp::CompletionItem> {
-        let mut completions = vec![];
-
-        // Prelude types
-        for type_ in PreludeType::iter() {
-            completions.push(lsp::CompletionItem {
-                label: type_.name().into(),
-                detail: Some("Type".into()),
-                kind: Some(lsp::CompletionItemKind::CLASS),
-                ..Default::default()
-            });
-        }
-
-        // Module types
-        for (name, type_) in &module.ast.type_info.types {
-            completions.push(type_completion(None, name, type_));
-        }
-
-        // Imported modules
-        for import in module.ast.definitions.iter().filter_map(get_import) {
-            // The module may not be known of yet if it has not previously
-            // compiled yet in this editor session.
-            // TODO: test getting completions from modules defined in other packages
-            let Some(module) = self.compiler.get_module_inferface(&import.module) else {
-                continue;
-            };
-
-            // Qualified types
-            for (name, type_) in &module.types {
-                if !self.is_suggestable_import(&type_.publicity, module.package.as_str()) {
-                    continue;
-                }
-
-                let module = import.used_name();
-                if module.is_some() {
-                    completions.push(type_completion(module.as_ref(), name, type_));
-                }
-            }
-
-            // Unqualified types
-            for unqualified in &import.unqualified_types {
-                match module.get_public_type(&unqualified.name) {
-                    Some(type_) => {
-                        completions.push(type_completion(None, unqualified.used_name(), type_))
-                    }
-                    None => continue,
-                }
-            }
-        }
-
-        completions
-    }
-
-    fn completion_values<'b>(&'b self, module: &'b Module) -> Vec<lsp::CompletionItem> {
-        let mut completions = vec![];
-
-        // Module functions
-        for (name, value) in &module.ast.type_info.values {
-            // Here we do not check for the internal attribute: we always want
-            // to show autocompletions for values defined in the same module,
-            // even if those are internal.
-            completions.push(value_completion(None, name, value));
-        }
-
-        // Imported modules
-        for import in module.ast.definitions.iter().filter_map(get_import) {
-            // The module may not be known of yet if it has not previously
-            // compiled yet in this editor session.
-            // TODO: test getting completions from modules defined in other packages
-            let Some(module) = self.compiler.get_module_inferface(&import.module) else {
-                continue;
-            };
-
-            // Qualified values
-            for (name, value) in &module.values {
-                if !self.is_suggestable_import(&value.publicity, module.package.as_str()) {
-                    continue;
-                }
-
-                let module = import.used_name();
-                if module.is_some() {
-                    completions.push(value_completion(module.as_deref(), name, value));
-                }
-            }
-
-            // Unqualified values
-            for unqualified in &import.unqualified_values {
-                match module.get_public_value(&unqualified.name) {
-                    Some(value) => {
-                        completions.push(value_completion(None, unqualified.used_name(), value))
-                    }
-                    None => continue,
-                }
-            }
-        }
-
-        completions
-    }
-
-    fn unqualified_completions_from_module<'b>(
-        &'b self,
-        importing_module: &'b ModuleInterface,
-        module: &'b Module,
-        // should type completions include the word "type" in the completion
-        include_type_in_completion: bool,
-    ) -> Vec<lsp::CompletionItem> {
-        let mut completions = vec![];
-
-        // Find values and type that have already previously been imported
-        let mut already_imported_types = std::collections::HashSet::new();
-        let mut already_imported_values = std::collections::HashSet::new();
-
-        // Search the ast for import statements
-        for import in module.ast.definitions.iter().filter_map(get_import) {
-            // Find the import that matches the module being imported from
-            if import.module == importing_module.name {
-                // Add the values and types that have already been imported
-                for unqualified in &import.unqualified_types {
-                    let _ = already_imported_types.insert(&unqualified.name);
-                }
-
-                for unqualified in &import.unqualified_values {
-                    let _ = already_imported_values.insert(&unqualified.name);
-                }
-            }
-        }
-
-        // Get completable types
-        for (name, type_) in &importing_module.types {
-            // Skip types that should not be suggested
-            if !self.is_suggestable_import(&type_.publicity, importing_module.package.as_str()) {
-                continue;
-            }
-
-            // Skip type that are already imported
-            if already_imported_types.contains(name) {
-                continue;
-            }
-
-            let completion: lsp::CompletionItem = if !include_type_in_completion {
-                type_completion(None, name, type_)
-            } else {
-                let completion = type_completion(None, name, type_);
-                lsp::CompletionItem {
-                    // Add type prior to unqualified import for types
-                    insert_text: Some("type ".to_string() + &completion.label),
-                    ..completion
-                }
-            };
-            completions.push(completion);
-        }
-
-        // Get completable values
-        for (name, value) in &importing_module.values {
-            // Skip values that should not be suggested
-            if !self.is_suggestable_import(&value.publicity, importing_module.package.as_str()) {
-                continue;
-            }
-
-            // Skip values that are already imported
-            if already_imported_values.contains(name) {
-                continue;
-            }
-            completions.push(value_completion(None, name, value));
-        }
-
-        completions
-    }
-
-    fn import_completions<'b>(
-        &'b self,
-        src: EcoString,
-        params: &lsp::TextDocumentPositionParams,
-        module: &'b Module,
-    ) -> Option<Result<Option<Vec<lsp::CompletionItem>>>> {
-        let line_num = LineNumbers::new(src.as_str());
-        let start_of_line = line_num.byte_index(params.position.line, 0);
-        let end_of_line = line_num.byte_index(params.position.line + 1, 0);
-
-        // Drop all lines except the line the cursor is on
-        let src = &src.get(start_of_line as usize..end_of_line as usize)?;
-
-        // If this isn't an import line then we don't offer import completions
-        if !src.trim_start().starts_with("import") {
-            return None;
-        }
-
-        // Check if we are completing an unqualified import
-        if let Some(dot_index) = src.find('.') {
-            // Find the module that is being imported from
-            let importing_module_name = src.get(6..dot_index)?.trim();
-            let importing_module: &ModuleInterface =
-                self.compiler.get_module_inferface(importing_module_name)?;
-
-            // Check if the cursor is proceeded by the word "type".
-            // We want to make sure suggestions don't include the word "type"
-            // if the cursor is proceeded by it.
-            let cursor = src.get(..params.position.character as usize)?;
-            Some(Ok(Some(self.unqualified_completions_from_module(
-                importing_module,
-                module,
-                !cursor.trim().ends_with("type"),
-            ))))
-        } else {
-            // Find where to start and end the import completion
-            let start = line_num.line_and_column_number(start_of_line);
-            let end = line_num.line_and_column_number(end_of_line - 1);
-            let start = lsp::Position::new(start.line - 1, start.column + 6);
-            let end = lsp::Position::new(end.line - 1, end.column - 1);
-            let completions = self.complete_modules_for_import(module, start, end);
-
-            Some(Ok(Some(completions)))
-        }
-    }
-
-    fn complete_modules_for_import<'b>(
-        &'b self,
-        current_module: &'b Module,
-        start: lsp::Position,
-        end: lsp::Position,
-    ) -> Vec<lsp::CompletionItem> {
-        let mut direct_dep_packages: std::collections::HashSet<&EcoString> =
-            std::collections::HashSet::from_iter(
-                self.compiler.project_compiler.config.dependencies.keys(),
-            );
-        if !current_module.origin.is_src() {
-            // In tests we can import direct dev dependencies
-            direct_dep_packages.extend(
-                self.compiler
-                    .project_compiler
-                    .config
-                    .dev_dependencies
-                    .keys(),
-            )
-        }
-
-        let already_imported: std::collections::HashSet<EcoString> =
-            std::collections::HashSet::from_iter(current_module.dependencies_list());
-        self.compiler
-            .project_compiler
-            .get_importable_modules()
-            .iter()
-            //
-            // It is possible to import modules from dependencies of dependencies
-            // but it's not recommended so we don't include them in completions
-            .filter(|(_, module)| {
-                let is_root_or_prelude =
-                    module.package == self.root_package_name() || module.package.is_empty();
-                is_root_or_prelude || direct_dep_packages.contains(&module.package)
-            })
-            //
-            // src/ cannot import test/
-            .filter(|(_, module)| module.origin.is_src() || !current_module.origin.is_src())
-            //
-            // It is possible to import internal modules from other packages,
-            // but it's not recommended so we don't include them in completions
-            .filter(|(_, module)| module.package == self.root_package_name() || !module.is_internal)
-            //
-            // You cannot import a module twice
-            .filter(|(name, _)| !already_imported.contains(*name))
-            //
-            // You cannot import yourself
-            .filter(|(name, _)| *name != &current_module.name)
-            //
-            // Everything else we suggest as a completion
-            .map(|(name, _)| lsp::CompletionItem {
-                label: name.to_string(),
-                kind: Some(lsp::CompletionItemKind::MODULE),
-                text_edit: Some(lsp::CompletionTextEdit::Edit(lsp::TextEdit {
-                    range: lsp::Range { start, end },
-                    new_text: name.to_string(),
-                })),
-                ..Default::default()
-            })
-            .collect()
-    }
-
-    fn root_package_name(&self) -> &str {
-        self.compiler.project_compiler.config.name.as_str()
-    }
-}
-
-fn type_completion(
-    module: Option<&EcoString>,
-    name: &str,
-    type_: &TypeConstructor,
-) -> lsp::CompletionItem {
-    let label = match module {
-        Some(module) => format!("{module}.{name}"),
-        None => name.to_string(),
-    };
-
-    let kind = Some(if type_.typ.is_variable() {
-        lsp::CompletionItemKind::VARIABLE
-    } else {
-        lsp::CompletionItemKind::CLASS
-    });
-
-    lsp::CompletionItem {
-        label,
-        kind,
-        detail: Some("Type".into()),
-        ..Default::default()
-    }
-}
-
-fn value_completion(
-    module: Option<&str>,
-    name: &str,
-    value: &crate::type_::ValueConstructor,
-) -> lsp::CompletionItem {
-    let label = match module {
-        Some(module) => format!("{module}.{name}"),
-        None => name.to_string(),
-    };
-
-    let type_ = Printer::new().pretty_print(&value.type_, 0);
-
-    let kind = Some(match value.variant {
-        ValueConstructorVariant::LocalVariable { .. } => lsp::CompletionItemKind::VARIABLE,
-        ValueConstructorVariant::ModuleConstant { .. } => lsp::CompletionItemKind::CONSTANT,
-        ValueConstructorVariant::LocalConstant { .. } => lsp::CompletionItemKind::CONSTANT,
-        ValueConstructorVariant::ModuleFn { .. } => lsp::CompletionItemKind::FUNCTION,
-        ValueConstructorVariant::Record { arity: 0, .. } => lsp::CompletionItemKind::ENUM_MEMBER,
-        ValueConstructorVariant::Record { .. } => lsp::CompletionItemKind::CONSTRUCTOR,
-    });
-
-    let documentation = value.get_documentation().map(|d| {
-        lsp::Documentation::MarkupContent(lsp::MarkupContent {
-            kind: lsp::MarkupKind::Markdown,
-            value: d.to_string(),
-        })
-    });
-
-    lsp::CompletionItem {
-        label,
-        kind,
-        detail: Some(type_),
-        documentation,
-        ..Default::default()
-    }
-}
-
-fn get_import(statement: &TypedDefinition) -> Option<&Import<EcoString>> {
-    match statement {
-        Definition::Import(import) => Some(import),
-        _ => None,
-    }
 }
 
 fn hover_for_pattern(pattern: &TypedPattern, line_numbers: LineNumbers) -> Hover {
